"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const VideoNativeComponent_1 = __importDefault(require("./specs/VideoNativeComponent"));
const utils_1 = require("./utils");
const NativeVideoManager_1 = __importDefault(require("./specs/NativeVideoManager"));
const types_1 = require("./types");
const Video = (0, react_1.forwardRef)(({ source, style, resizeMode, posterResizeMode, poster, drm, textTracks, selectedVideoTrack, selectedAudioTrack, selectedTextTrack, useTextureView, useSecureView, viewType, shutterColor, onLoadStart, onLoad, onError, onProgress, onSeek, onEnd, onBuffer, onBandwidthUpdate, onControlsVisibilityChange, onExternalPlaybackChange, onFullscreenPlayerWillPresent, onFullscreenPlayerDidPresent, onFullscreenPlayerWillDismiss, onFullscreenPlayerDidDismiss, onReadyForDisplay, onPlaybackRateChange, onVolumeChange, onAudioBecomingNoisy, onPictureInPictureStatusChanged, onRestoreUserInterfaceForPictureInPictureStop, onReceiveAdEvent, onPlaybackStateChanged, onAudioFocusChanged, onIdle, onTimedMetadata, onAudioTracks, onTextTracks, onTextTrackDataChanged, onVideoTracks, onAspectRatio, ...rest }, ref) => {
    const nativeRef = (0, react_1.useRef)(null);
    const [showPoster, setShowPoster] = (0, react_1.useState)(!!poster);
    const [_restoreUserInterfaceForPIPStopCompletionHandler, setRestoreUserInterfaceForPIPStopCompletionHandler,] = (0, react_1.useState)();
    const hasPoster = !!poster;
    const posterStyle = (0, react_1.useMemo)(() => ({
        ...react_native_1.StyleSheet.absoluteFillObject,
        resizeMode: posterResizeMode && posterResizeMode !== 'none'
            ? posterResizeMode
            : 'contain',
    }), [posterResizeMode]);
    const src = (0, react_1.useMemo)(() => {
        if (!source) {
            return undefined;
        }
        const resolvedSource = (0, utils_1.resolveAssetSourceForVideo)(source);
        let uri = resolvedSource.uri || '';
        if (uri && uri.match(/^\//)) {
            uri = `file://${uri}`;
        }
        if (!uri) {
            console.log('Trying to load empty source');
        }
        const isNetwork = !!(uri && uri.match(/^(rtp|rtsp|http|https):/));
        const isAsset = !!(uri &&
            uri.match(/^(assets-library|ipod-library|file|content|ms-appx|ms-appdata):/));
        const selectedDrm = source.drm || drm;
        const _drm = !selectedDrm
            ? undefined
            : {
                type: selectedDrm.type,
                licenseServer: selectedDrm.licenseServer,
                headers: (0, utils_1.generateHeaderForNative)(selectedDrm.headers),
                contentId: selectedDrm.contentId,
                certificateUrl: selectedDrm.certificateUrl,
                base64Certificate: selectedDrm.base64Certificate,
                useExternalGetLicense: !!selectedDrm.getLicense,
                multiDrm: selectedDrm.multiDrm,
            };
        return {
            uri,
            isNetwork,
            isAsset,
            shouldCache: resolvedSource.shouldCache || false,
            type: resolvedSource.type || '',
            mainVer: resolvedSource.mainVer || 0,
            patchVer: resolvedSource.patchVer || 0,
            requestHeaders: (0, utils_1.generateHeaderForNative)(resolvedSource.headers),
            startPosition: resolvedSource.startPosition ?? -1,
            cropStart: resolvedSource.cropStart || 0,
            cropEnd: resolvedSource.cropEnd,
            metadata: resolvedSource.metadata,
            drm: _drm,
            textTracksAllowChunklessPreparation: resolvedSource.textTracksAllowChunklessPreparation,
        };
    }, [drm, source]);
    const _selectedTextTrack = (0, react_1.useMemo)(() => {
        if (!selectedTextTrack) {
            return;
        }
        const typeOfValueProp = typeof selectedTextTrack.value;
        if (typeOfValueProp !== 'number' &&
            typeOfValueProp !== 'string' &&
            typeOfValueProp !== 'undefined') {
            console.warn('invalid type provided to selectedTextTrack.value: ', typeOfValueProp);
            return;
        }
        return {
            type: selectedTextTrack?.type,
            value: `${selectedTextTrack.value}`,
        };
    }, [selectedTextTrack]);
    const _selectedAudioTrack = (0, react_1.useMemo)(() => {
        if (!selectedAudioTrack) {
            return;
        }
        const typeOfValueProp = typeof selectedAudioTrack.value;
        if (typeOfValueProp !== 'number' &&
            typeOfValueProp !== 'string' &&
            typeOfValueProp !== 'undefined') {
            console.warn('invalid type provided to selectedAudioTrack.value: ', typeOfValueProp);
            return;
        }
        return {
            type: selectedAudioTrack?.type,
            value: `${selectedAudioTrack.value}`,
        };
    }, [selectedAudioTrack]);
    const _selectedVideoTrack = (0, react_1.useMemo)(() => {
        if (!selectedVideoTrack) {
            return;
        }
        const typeOfValueProp = typeof selectedVideoTrack.value;
        if (typeOfValueProp !== 'number' &&
            typeOfValueProp !== 'string' &&
            typeOfValueProp !== 'undefined') {
            console.warn('invalid type provided to selectedVideoTrack.value: ', typeOfValueProp);
            return;
        }
        return {
            type: selectedVideoTrack?.type,
            value: `${selectedVideoTrack.value}`,
        };
    }, [selectedVideoTrack]);
    const seek = (0, react_1.useCallback)(async (time, tolerance) => {
        if (isNaN(time) || time === null) {
            throw new Error("Specified time is not a number: '" + time + "'");
        }
        if (!nativeRef.current) {
            console.warn('Video Component is not mounted');
            return;
        }
        const callSeekFunction = () => {
            NativeVideoManager_1.default.seekCmd((0, utils_1.getReactTag)(nativeRef), time, tolerance || 0);
        };
        react_native_1.Platform.select({
            ios: callSeekFunction,
            android: callSeekFunction,
            default: () => {
                // TODO: Implement VideoManager.seekCmd for windows
                nativeRef.current?.setNativeProps({ seek: time });
            },
        })();
    }, []);
    const pause = (0, react_1.useCallback)(() => {
        return NativeVideoManager_1.default.setPlayerPauseStateCmd((0, utils_1.getReactTag)(nativeRef), true);
    }, []);
    const resume = (0, react_1.useCallback)(() => {
        return NativeVideoManager_1.default.setPlayerPauseStateCmd((0, utils_1.getReactTag)(nativeRef), false);
    }, []);
    const setVolume = (0, react_1.useCallback)((volume) => {
        return NativeVideoManager_1.default.setVolumeCmd((0, utils_1.getReactTag)(nativeRef), volume);
    }, []);
    const setFullScreen = (0, react_1.useCallback)((fullScreen) => {
        return NativeVideoManager_1.default.setFullScreenCmd((0, utils_1.getReactTag)(nativeRef), fullScreen);
    }, []);
    const presentFullscreenPlayer = (0, react_1.useCallback)(() => setFullScreen(true), [setFullScreen]);
    const dismissFullscreenPlayer = (0, react_1.useCallback)(() => setFullScreen(false), [setFullScreen]);
    const save = (0, react_1.useCallback)((options) => {
        // VideoManager.save can be null on android & windows
        if (react_native_1.Platform.OS !== 'ios') {
            return;
        }
        // @todo Must implement it in a different way.
        return NativeVideoManager_1.default.save?.((0, utils_1.getReactTag)(nativeRef), options);
    }, []);
    const getCurrentPosition = (0, react_1.useCallback)(() => {
        // @todo Must implement it in a different way.
        return NativeVideoManager_1.default.getCurrentPosition((0, utils_1.getReactTag)(nativeRef));
    }, []);
    const restoreUserInterfaceForPictureInPictureStopCompleted = (0, react_1.useCallback)((restored) => {
        setRestoreUserInterfaceForPIPStopCompletionHandler(restored);
    }, [setRestoreUserInterfaceForPIPStopCompletionHandler]);
    const onVideoLoadStart = (0, react_1.useCallback)((e) => {
        hasPoster && setShowPoster(true);
        onLoadStart?.(e.nativeEvent);
    }, [hasPoster, onLoadStart]);
    const onVideoLoad = (0, react_1.useCallback)((e) => {
        if (react_native_1.Platform.OS === 'windows') {
            hasPoster && setShowPoster(false);
        }
        onLoad?.(e.nativeEvent);
    }, [onLoad, hasPoster, setShowPoster]);
    const onVideoError = (0, react_1.useCallback)((e) => {
        onError?.(e.nativeEvent);
    }, [onError]);
    const onVideoProgress = (0, react_1.useCallback)((e) => {
        onProgress?.(e.nativeEvent);
    }, [onProgress]);
    const onVideoSeek = (0, react_1.useCallback)((e) => {
        onSeek?.(e.nativeEvent);
    }, [onSeek]);
    const onVideoPlaybackStateChanged = (0, react_1.useCallback)((e) => {
        onPlaybackStateChanged?.(e.nativeEvent);
    }, [onPlaybackStateChanged]);
    const _shutterColor = (0, react_1.useMemo)(() => {
        const color = (0, react_native_1.processColor)(shutterColor);
        return typeof color === 'number' ? color : undefined;
    }, [shutterColor]);
    // android only
    const _onTimedMetadata = (0, react_1.useCallback)((e) => {
        onTimedMetadata?.(e.nativeEvent);
    }, [onTimedMetadata]);
    const _onAudioTracks = (0, react_1.useCallback)((e) => {
        onAudioTracks?.(e.nativeEvent);
    }, [onAudioTracks]);
    const _onTextTracks = (0, react_1.useCallback)((e) => {
        onTextTracks?.(e.nativeEvent);
    }, [onTextTracks]);
    const _onTextTrackDataChanged = (0, react_1.useCallback)((e) => {
        const { ...eventData } = e.nativeEvent;
        delete eventData.target;
        onTextTrackDataChanged?.(eventData);
    }, [onTextTrackDataChanged]);
    const _onVideoTracks = (0, react_1.useCallback)((e) => {
        onVideoTracks?.(e.nativeEvent);
    }, [onVideoTracks]);
    const _onPlaybackRateChange = (0, react_1.useCallback)((e) => {
        onPlaybackRateChange?.(e.nativeEvent);
    }, [onPlaybackRateChange]);
    const _onVolumeChange = (0, react_1.useCallback)((e) => {
        onVolumeChange?.(e.nativeEvent);
    }, [onVolumeChange]);
    const _onReadyForDisplay = (0, react_1.useCallback)(() => {
        hasPoster && setShowPoster(false);
        onReadyForDisplay?.();
    }, [setShowPoster, hasPoster, onReadyForDisplay]);
    const _onPictureInPictureStatusChanged = (0, react_1.useCallback)((e) => {
        onPictureInPictureStatusChanged?.(e.nativeEvent);
    }, [onPictureInPictureStatusChanged]);
    const _onAudioFocusChanged = (0, react_1.useCallback)((e) => {
        onAudioFocusChanged?.(e.nativeEvent);
    }, [onAudioFocusChanged]);
    const onVideoBuffer = (0, react_1.useCallback)((e) => {
        onBuffer?.(e.nativeEvent);
    }, [onBuffer]);
    const onVideoExternalPlaybackChange = (0, react_1.useCallback)((e) => {
        onExternalPlaybackChange?.(e.nativeEvent);
    }, [onExternalPlaybackChange]);
    const _onBandwidthUpdate = (0, react_1.useCallback)((e) => {
        onBandwidthUpdate?.(e.nativeEvent);
    }, [onBandwidthUpdate]);
    const _onReceiveAdEvent = (0, react_1.useCallback)((e) => {
        onReceiveAdEvent?.(e.nativeEvent);
    }, [onReceiveAdEvent]);
    const _onVideoAspectRatio = (0, react_1.useCallback)((e) => {
        onAspectRatio?.(e.nativeEvent);
    }, [onAspectRatio]);
    const _onControlsVisibilityChange = (0, react_1.useCallback)((e) => {
        onControlsVisibilityChange?.(e.nativeEvent);
    }, [onControlsVisibilityChange]);
    const usingExternalGetLicense = drm?.getLicense instanceof Function;
    const onGetLicense = (0, react_1.useCallback)(async (event) => {
        if (!usingExternalGetLicense) {
            return;
        }
        const data = event.nativeEvent;
        let result;
        if (data?.spcBase64) {
            try {
                // Handles both scenarios, getLicenseOverride being a promise and not.
                const license = await drm.getLicense(data.spcBase64, data.contentId, data.licenseUrl, data.loadedLicenseUrl);
                result =
                    typeof license === 'string' ? license : 'Empty license result';
            }
            catch {
                result = 'fetch error';
            }
        }
        else {
            result = 'No spc received';
        }
        if (nativeRef.current) {
            NativeVideoManager_1.default.setLicenseResultErrorCmd((0, utils_1.getReactTag)(nativeRef), result, data.loadedLicenseUrl);
        }
    }, [drm, usingExternalGetLicense]);
    (0, react_1.useImperativeHandle)(ref, () => ({
        seek,
        presentFullscreenPlayer,
        dismissFullscreenPlayer,
        save,
        pause,
        resume,
        restoreUserInterfaceForPictureInPictureStopCompleted,
        setVolume,
        getCurrentPosition,
        setFullScreen,
    }), [
        seek,
        presentFullscreenPlayer,
        dismissFullscreenPlayer,
        save,
        pause,
        resume,
        restoreUserInterfaceForPictureInPictureStopCompleted,
        setVolume,
        getCurrentPosition,
        setFullScreen,
    ]);
    const _viewType = (0, react_1.useMemo)(() => {
        const hasValidDrmProp = drm !== undefined && Object.keys(drm).length !== 0;
        const shallForceViewType = hasValidDrmProp && (viewType === types_1.ViewType.TEXTURE || useTextureView);
        if (shallForceViewType) {
            console.warn('cannot use DRM on texture view. please set useTextureView={false}');
        }
        if (useSecureView && useTextureView) {
            console.warn('cannot use SecureView on texture view. please set useTextureView={false}');
        }
        return shallForceViewType
            ? useSecureView
                ? types_1.ViewType.SURFACE_SECURE
                : types_1.ViewType.SURFACE // check if we should force the type to Surface due to DRM
            : viewType
                ? viewType // else use ViewType from source
                : useSecureView // else infer view type from useSecureView and useTextureView
                    ? types_1.ViewType.SURFACE_SECURE
                    : useTextureView
                        ? types_1.ViewType.TEXTURE
                        : types_1.ViewType.SURFACE;
    }, [drm, useSecureView, useTextureView, viewType]);
    return (react_1.default.createElement(react_native_1.View, { style: style },
        react_1.default.createElement(VideoNativeComponent_1.default, { ref: nativeRef, ...rest, src: src, style: react_native_1.StyleSheet.absoluteFill, resizeMode: resizeMode, restoreUserInterfaceForPIPStopCompletionHandler: _restoreUserInterfaceForPIPStopCompletionHandler, textTracks: textTracks, selectedTextTrack: _selectedTextTrack, selectedAudioTrack: _selectedAudioTrack, selectedVideoTrack: _selectedVideoTrack, shutterColor: _shutterColor, onGetLicense: usingExternalGetLicense ? onGetLicense : undefined, onVideoLoad: onLoad || hasPoster
                ? onVideoLoad
                : undefined, onVideoLoadStart: onLoadStart || hasPoster ? onVideoLoadStart : undefined, onVideoError: onError ? onVideoError : undefined, onVideoProgress: onProgress ? onVideoProgress : undefined, onVideoSeek: onSeek ? onVideoSeek : undefined, onVideoEnd: onEnd, onVideoBuffer: onBuffer ? onVideoBuffer : undefined, onVideoPlaybackStateChanged: onPlaybackStateChanged ? onVideoPlaybackStateChanged : undefined, onVideoBandwidthUpdate: onBandwidthUpdate ? _onBandwidthUpdate : undefined, onTimedMetadata: onTimedMetadata ? _onTimedMetadata : undefined, onAudioTracks: onAudioTracks ? _onAudioTracks : undefined, onTextTracks: onTextTracks ? _onTextTracks : undefined, onTextTrackDataChanged: onTextTrackDataChanged ? _onTextTrackDataChanged : undefined, onVideoTracks: onVideoTracks ? _onVideoTracks : undefined, onVideoFullscreenPlayerDidDismiss: onFullscreenPlayerDidDismiss, onVideoFullscreenPlayerDidPresent: onFullscreenPlayerDidPresent, onVideoFullscreenPlayerWillDismiss: onFullscreenPlayerWillDismiss, onVideoFullscreenPlayerWillPresent: onFullscreenPlayerWillPresent, onVideoExternalPlaybackChange: onExternalPlaybackChange ? onVideoExternalPlaybackChange : undefined, onVideoIdle: onIdle, onAudioFocusChanged: onAudioFocusChanged ? _onAudioFocusChanged : undefined, onReadyForDisplay: onReadyForDisplay || hasPoster ? _onReadyForDisplay : undefined, onPlaybackRateChange: onPlaybackRateChange ? _onPlaybackRateChange : undefined, onVolumeChange: onVolumeChange ? _onVolumeChange : undefined, onVideoAudioBecomingNoisy: onAudioBecomingNoisy, onPictureInPictureStatusChanged: onPictureInPictureStatusChanged
                ? _onPictureInPictureStatusChanged
                : undefined, onRestoreUserInterfaceForPictureInPictureStop: onRestoreUserInterfaceForPictureInPictureStop, onVideoAspectRatio: onAspectRatio ? _onVideoAspectRatio : undefined, onReceiveAdEvent: onReceiveAdEvent
                ? _onReceiveAdEvent
                : undefined, onControlsVisibilityChange: onControlsVisibilityChange ? _onControlsVisibilityChange : undefined, viewType: _viewType }),
        hasPoster && showPoster ? (react_1.default.createElement(react_native_1.Image, { style: posterStyle, source: { uri: poster } })) : null));
});
Video.displayName = 'Video';
exports.default = Video;
//# sourceMappingURL=Video.js.map